/*
 *             High performance packet classification 
 *                     <http://www.hipac.org>
 *
 * (c) 2004-2005   MARA Systems AB <http://www.marasystems.com>
 *                 +-----------------------------+
 *                 |       Michael Bellion       |
 *                 |  <michael@marasystems.com>  |
 *                 +-----------------------------+
 *
 * (c) 2002-2003   hipac core team <nf@hipac.org>:
 *     +---------------------------+--------------------------+
 *     |      Michael Bellion      |       Thomas Heinz       |
 *     |   <mbellion@hipac.org>    |   <creatix@hipac.org>    |
 *     +---------------------------+--------------------------+
 *
 * Licenced under the GNU General Public Licence, version 2.
 */


#ifndef _LIBNFHIPAC_H
#define _LIBNFHIPAC_H

#include <features.h>
#if defined(__GLIBC__) && __GLIBC__ == 2
#  include <net/if.h>
#  include <netinet/in.h>
#  include <asm/types.h>
#else /* libc5 */
#  include <linux/if.h>
#  include <linux/in.h>
#  include <linux/types.h>
#endif
#include "nfhp_com.h"

/*
  return empty command struct s where only the command type is initialized by
  cmd; the size of s which is at least (127 KB) - (NLMSG_LENGTH(0) bytes) is
  stored in *size if size is not NULL; on error NULL is returned;
  IMPORTANT: __never__ call free, realloc or similar functions for s;
             use nlhp_free_cmd instead to free the memory allocated for s
             by nlhp_new_cmd;
             realloc is never needed because the size of s is chosen to
             be large enough (don't forget that the size of skb->data is
	     limited to 128 KB)
*/
struct nfhp_cmd *
nlhp_new_cmd(int cmd, int *size);

/* free cmd which must have been allocated by nlhp_new_cmd */
void
nlhp_free_cmd(struct nfhp_cmd *cmd);

/* returns a copy of cmd; the size of cmd is passed via size */
struct nfhp_cmd *
nlhp_copy_cmd(struct nfhp_cmd *cmd, int size);

/*
  send command request to nf_hipac kernel module (necessary parts in cmd
  must be filled in appropriately since cmd is not checked or modified by
  the library); the size of cmd is passed via cmd_size; if cmd->cmd is
  CMD_LIST then process_list must point to a callback function which
  is called for each packet received during a list request;
  otherwise process_list may be NULL; when called for the first time
  process_list can assume that data points to a struct nfhp_list_chain;
  process_list should return 0 on success and a value < 0 on error;
  each packet may contain several struct nfhp_list_chain and struct
  nfhp_list_rule arranged in the way described in nfhp_com.h;
  
  if nlhp_send_cmd returns 0 then see nlhp_error(*err) for an error
  description; *err == 0 indicates a successful operation;
    
  return values:  0: see *err
                 -1: system error (see perror or errno)
		 -2: read timeout
		 -3: not enough memory available
		 -4: process_list failed
		 -5: other error
*/
int
nlhp_send_cmd(struct nfhp_cmd *cmd, int cmd_size,
	      int (*process_list) (const void *data, int len), int *err);

/*
 * return string description of err if available; otherwise NULL is returned;
 */
const char *
nlhp_error(int err);

#endif
